import sys

from dokku_client import client
from dokku_client.commands import BaseCommand
from dokku_client.command_tools import command_by_name, root_doc

class HelpCommand(BaseCommand):
    """Show this help message

    usage:
        dokku-client help [COMMAND]
    """
    check_config = False
    sort_order = 5
    
    def main(self):
        if not self.args['COMMAND']:
            print root_doc()
        else:
            command = command_by_name(self.args['COMMAND'])
            if not command:
                sys.stderr.write("Unknown command. Use 'dokku-client help' for list of commands.\n")
            else:
                print command.doc



#! /usr/bin/env python
"""Client for Dokku

usage:
    dokku-client <command> [<args>...]
    dokku-client help

global options:
    -H <host>, --host=<host>      Host address
    -a <app>, --app=<app>         App name

----

See 'git help <command>' for more information on a specific command.
"""
import sys
from subprocess import call

from docopt import docopt

from dokku_client import __version__
from dokku_client.command_tools import load_commands, command_by_name, apply_defaults, root_doc


def main():
    commands = load_commands()

    args = docopt(root_doc(), version='dokku-client version %s' % __version__, options_first=True)
    command_name = args['<command>'] or 'help'

    if command_name == 'version':
        # docopt will handle version printing for us if use '--version'
        exit(call(['dokku-client', '--version']))

    # Get the command object for the specified command name
    command = command_by_name(command_name, commands)
    if not command:
        sys.stderr.write("Unknown command. Use 'dokku-client help' for list of commands.\n")
        exit(1)
    else:
        # Use docopt to parse the options based upon the class' doc string
        command_args = docopt(command.doc)
        # Load default values from the users' environment
        command_args = apply_defaults(command_args)
        if command.check_config:
            # Sanity check the config
            if not command_args.get('--host', None):
                sys.stderr.write("Could not determine host. Specify --host or set DOKKU_HOST.\n")
                exit(1)
            if not command_args.get('--app', None):
                sys.stderr.write("Could not determine app. Specify --app or set DOKKU_APP.\n")
                exit(1)
        # Ok, let's run the command
        command.args = command_args
        command.main()

if __name__ == '__main__':
    main()


from subprocess import call

from dokku_client.commands import BaseCommand

class PromptCommand(BaseCommand):
    """Open a prompt

    usage:
        dokku-client prompt [options]

    global options:
        [[include global options]]
    """
    
    def main(self):
        cmd = 'docker run -i -t app/%s /bin/bash' % self.args['--app']
        call_args = ['ssh', '-t', self.args['--host'], cmd]
        print "Running command: %s" % ' '.join(call_args)
        call(call_args)



import re
import os
import pkg_resources

def load_commands():
    commands = []
    for entry_point in pkg_resources.iter_entry_points(group='dokku_client.commands'):
        command_class = entry_point.load()
        command = command_class(name=entry_point.name)
        commands.append(command)
    commands = sorted(commands, key=lambda c: (c.sort_order, c.name))
    return commands

def command_list_doc(commands=None):
    commands = commands or load_commands()

    doc = "full list of available commands:\n\n"
    names = []
    descriptions = []

    for command in commands:
        names.append(command.name)
        descriptions.append(command.description)

    name_col_length = max(len(n) for n in names)
    for name, description in zip(names, descriptions):
        doc += "    %s  %s\n" % (name.ljust(name_col_length), description)

    return doc.strip()

def root_doc(commands=None):
    from dokku_client import client
    doc = client.__doc__.replace('----', command_list_doc())
    return doc.strip()


def command_by_name(name, commands=None):
    commands = commands or load_commands()
    for command in commands:
        if command.name == name:
            return command
    return None

def global_opts_doc():
    from dokku_client import client
    lines = client.__doc__.split("\n")
    lines = filter(lambda l: re.match(r'\s*-[a-zA-Z].*', l), lines)
    return "\n".join(lines)

def apply_defaults(args):
    for key, val in args.items():
        if not val:
            env_var = 'DOKKU_%s' % key.replace('--', '').upper()
            if os.environ.get(env_var):
                args[key] = os.environ.get(env_var).strip()
    return args





import re

from sarge import capture_stdout, run

from dokku_client.command_tools import global_opts_doc

class BaseCommand(object):
    check_config = True
    sort_order = 10

    def __init__(self, name):
        self._name = name
        # Set externally in client.py
        self.args = {}

    @property
    def doc(self):
        """Get the docs for this command

        The docs are used for display purposes, but also by 
        docopt (which uses them to the parse command line options)
        """

        doc = self.__doc__
        # Unindent by one level
        doc = re.sub(r'^( {4}|\t)', '',  doc, flags=re.MULTILINE)
        # Put the global options in place
        doc = re.sub(r'\s+\[\[include global options\]\]', "\n\n%s" % global_opts_doc(), doc)
        return doc

    @property
    def description(self):
        return self.__doc__.strip().split("\n")[0]

    @property
    def name(self):
        return self._name

    def main(self, global_args, command_args):
        raise NotImplementedError('Implement the main() method to implement your command')

    def run_remote(self, cmd, input=None):
        host = self.args['--host']
        return capture_stdout('ssh %s -- %s' % (host, cmd), input=input)

    def restart_container(self):
        app = self.args['--app']
        self.run_remote('docker restart `cat /home/git/%s/CONTAINER`' % app)



import re

from dokku_client.commands import BaseCommand

class ConfigSetCommand(BaseCommand):
    """Set one or more config options in the app's ENV file

    usage:
        dokku-client configset [options] (<KEY>=<VALUE>)...

    global options:
        [[include global options]]
    """

    def main(self):
        key_values = dict([kv.split('=') for kv in self.args['<KEY>=<VALUE>']])
        app = self.args['--app']

        self.run_remote('sudo -u git touch /home/git/%s/ENV' % app)
        p = self.run_remote('sudo -u git cat /home/git/%s/ENV' % app)
        env = p.stdout.text.strip()
        lines = env.split("\n")
        # For each line in the form 'export KEY=...'
        for i, line in enumerate(lines):
            matches = re.match('^export (.*?)=', line)
            if matches:
                key = matches.group(1)
                # If the key has been specified, then replace the line
                if key in key_values:
                    lines[i] = 'export %s="%s"' % (key, key_values[key])
                    del key_values[key]

        # Now append the remaining lnes
        for key, value in key_values.items():
            lines.append('export %s=%s' % (key, value))

        env = "\n".join(lines)
        self.run_remote('sudo -u git tee /home/git/python-django-sample/ENV', input=env)

        print "Config set, resting container"
        self.restart_container()


class ConfigGetCommand(BaseCommand):
    """Set one or more config options

    usage:
        dokku-client configget [options]

    global options:
        [[include global options]]
    """

    def main(self):
        app = self.args['--app']
        p = self.run_remote('sudo -u git cat /home/git/%s/ENV' % app)
        env = p.stdout.text.strip()
        print env


from subprocess import call

from dokku_client.commands import BaseCommand

class RestartCommand(BaseCommand):
    """Restart the container

    usage:
        dokku-client restart [options]

    global options:
        [[include global options]]
    """
    
    def main(self):
        self.restart_container()



#!/usr/bin/env python

from os.path import exists
from setuptools import setup, find_packages

from dokku_client import __version__

setup(
    name='dokku-client',
    version=__version__,
    author='Adam Charnock',
    author_email='adam@adamcharnock.com',
    packages=find_packages(),
    scripts=[],
    url='https://github.com/adamcharnock/dokku-client',
    license='MIT',
    description='Heroku-style command line interface for Dokku',
    long_description=open('README.rst').read() if exists("README.rst") else "",
    install_requires=[
        'docopt>=0.6.1',
        'sarge==0.1.1'
    ],
    entry_points={
        'dokku_client.commands': [
            'prompt = dokku_client.commands.prompt:PromptCommand',
            'help = dokku_client.commands.help:HelpCommand',
            'configset = dokku_client.commands.config:ConfigSetCommand',
            'configget = dokku_client.commands.config:ConfigGetCommand',
            'restart = dokku_client.commands.restart:RestartCommand',
        ],
        'console_scripts': [
            'dokku-client = dokku_client.client:main',
        ]
    }
)
